#include <stdio.h>
#include <memory.h>
#include <stdlib.h>
#include <time.h>
#include "raster.h"
#include "org_ecoinformatics_seek_gis_grass_RasterJniGlue.h"

extern int read_input(char *vector_map,int num_pnts);
extern void set_gridsize(int nrows, int ncols);
extern int set_boundary(double xmin,double ymin,double xmax,double ymax);
extern double* get_boundary();
extern int perform_rasterization();
extern int write_output(char *raster_map);

struct Cell_head region;
CELL **cell;
double *x=NULL,*y=NULL;
double xmin,ymin,xmax,ymax;
int npnts=0;

int read_input(char *vector_map,int num_pnts)
{
	FILE *fv;
	if((fv=fopen(vector_map,"r"))==NULL)
	{
		printf("can not open vector map file %s\n",vector_map);
		return(-11);
	}
	
	npnts=num_pnts;

	x=(double *)calloc(npnts,sizeof(double));
	y=(double *)calloc(npnts,sizeof(double));

	if(x==NULL || y==NULL)
		return (-12);
	
	double xx,yy;
	xmin=1e+10;
	ymin=1e+10;
	xmax=-1e+10;
	ymax=-1e+10;
	for(int i=0;i<npnts;i++)
	{
		fscanf(fv,"%lf %lf",&xx,&yy);
		x[i]=xx;
		y[i]=yy;
		if(xmin>xx) xmin=xx;
		if(ymin>yy) ymin=yy;
		if(xmax<xx) xmax=xx;
		if(ymax<yy) ymax=yy;
	}
	fclose(fv);
	return(0);
}

void set_gridsize(int nrows, int ncols)
{
	region.cols=ncols;
	region.rows=nrows;
	region.east=xmin;
	region.west=xmax;
	region.south=ymin;
	region.north=ymax;
	region.proj=PROJECTION_XY;
	region.ew_res=(xmax-xmin)/ncols;
	region.ns_res=(ymax-ymin)/nrows;
}

int set_boundary(double xmin,double ymin,double xmax,double ymax)
{
	if(region.cols<=0||region.rows<=0)
		return(-21);
	region.east=xmin;
	region.west=xmax;
	region.south=ymin;
	region.north=ymax;
	region.ew_res=(xmax-xmin)/region.cols;
	region.ns_res=(ymax-ymin)/region.rows;
	return(0);
}

double* get_boundary()
{
	//printf("%10.5f %1p.5f %10.5f %10.5f\n",xmin,ymin,xmax,ymax);
	static double ret[4]={xmin,ymin,xmax,ymax};
	return(ret);
}
int perform_rasterization()
{
	
	if(region.cols<=0||region.rows<=0)
		return(-31);
	if(npnts<=0)
		return(-32);
	
	//clock_t tb=clock();
	begin_rasterization(region.rows, region.cols);
	do_rasterization(region.rows, region.cols,x,y,npnts);
	/*printf("Rasterization (not include output) takes %10.3f seconds\n",(double)(clock()-tb)/CLOCKS_PER_SEC);
	getch();
	exit(0);*/
	return(0);
}

int write_output(char *raster_map)
{
	FILE *fr;
	if((fr=fopen(raster_map,"w"))==NULL)
	{
		printf("can not open raster map file %s\n",raster_map);
		return(-41);
	}
	/* ****************FIXME**************************/
	for(int i=0;i<region.rows;i++)
	{
    printf("i: %i\n", i);
		for(int j=0;j<region.cols;j++)
			fprintf(fr,"%3d",cell[i][j]);
		fprintf(fr,"\n");
	}
	fclose(fr);
	if(x!=NULL) free(x);
	if(y!=NULL) free(y);
	return(0);
}


/*int rasterization_implicit_boundary(char *vector_map,int num_pnts,char *raster_file,\
									int nrows, int ncols)
{
  int ret=read_input(vector_map,num_pnts,nrows,ncols);
  if(ret<0)	 return(ret);
  ret=perform_rasterization();
  if(ret<0) return(ret);
  ret=write_output(raster_file);
  if(ret<0) return(ret);
  return(0);
}


int rasterization_explicit_boundary(char *vector_map,int num_pnts,char *raster_file,\
									int nrows, int ncols, double xmin,double ymin,\
									double xmax,double ymax)
{
  int ret=read_input(vector_map,num_pnts,nrows,ncols,value);
  if(ret<0)	 return(ret);
  ret=set_boundary(xmin,ymin,xmax,ymax);
  if(ret<0) return(ret);
  ret=perform_rasterization();
  if(ret<0) return(ret);
  ret=write_output(raster_file);
  if(ret<0) return(ret);
  return(0);
}


JNIEXPORT jint JNICALL Java_RasterJniGlue_GISRaster
  (JNIEnv *env, jobject, jstring s1, jint i1, jstring s2, jint i2, jint i3)
{
  printf("Calling GISRaster in CPP Code.\n");
  jboolean iscopy;
  char * pfile = (char*) env->GetStringUTFChars(s1, &iscopy);
  char * rfile = (char*) env->GetStringUTFChars(s2, &iscopy);
  printf("calling GISRaster now\n");
  int ret=rasterization_implicit_boundary(pfile,i1,rfile,i2,i3,i4); 
  printf("DONE Calling GISRaster in CPP Code.\n");
  return(ret);
}*/


JNIEXPORT jint JNICALL Java_org_ecoinformatics_seek_gis_grass_RasterJniGlue_ReadInput
  (JNIEnv * env, jobject, jstring s1, jint i1)
{
	jboolean iscopy;
	char * pfile = (char*) env->GetStringUTFChars(s1, &iscopy);
	jint ret=read_input(pfile,i1);
	return(ret);
}


JNIEXPORT void JNICALL Java_org_ecoinformatics_seek_gis_grass_RasterJniGlue_SetGridSize
  (JNIEnv * env, jobject, jint i1, jint i2)
{
	set_gridsize(i1,i2);
}

JNIEXPORT jdoubleArray JNICALL Java_org_ecoinformatics_seek_gis_grass_RasterJniGlue_GetBoundary
  (JNIEnv * env, jobject)
{
	 printf("%10.5f %10.5f %10.5f %10.5f\n",xmin,ymin,xmax,ymax);
	 jdoubleArray ret = env->NewDoubleArray((jint)4) ;
	 double *OutData = env->GetDoubleArrayElements(ret,0); 
	 OutData[0]=xmin;
	 OutData[1]=ymin;
	 OutData[2]=xmax;
	 OutData[3]=ymax;
	 env->ReleaseDoubleArrayElements(ret, OutData, 0);
	 return(ret);
}

JNIEXPORT jint JNICALL Java_org_ecoinformatics_seek_gis_grass_RasterJniGlue_SetBoundary
  (JNIEnv *, jobject, jdouble d1, jdouble d2, jdouble d3, jdouble d4)
{
	jint ret=set_boundary(d1,d2,d3,d4);
	return(ret);
}

JNIEXPORT jint JNICALL Java_org_ecoinformatics_seek_gis_grass_RasterJniGlue_PerformRasterization
  (JNIEnv *, jobject)
{
	jint ret=perform_rasterization();
	return(ret);
}

JNIEXPORT jint JNICALL Java_org_ecoinformatics_seek_gis_grass_RasterJniGlue_WriteOutput
  (JNIEnv * env, jobject, jstring s1)
{
	jboolean iscopy;
	char * rfile = (char*) env->GetStringUTFChars(s1, &iscopy);
	jint ret=write_output(rfile);
	return(ret);
}

