/**
 *    '$RCSfile: OrderedResourceBundleTest.java,v $'
 *
 *     '$Author: brooke $'
 *       '$Date: 2006/03/02 22:39:16 $'
 *   '$Revision: 1.5 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package test.ptolemy.util;

import junit.framework.*;
import ptolemy.util.*;
import java.io.*;
import java.util.*;


/**
 * <p>test for ptolemy.util.OrderedResourceBundle class</p>
 *
 * @author Matthew Brooke
 * @version 1.0
 */
public class OrderedResourceBundleTest extends TestCase {

  private final String propsFileBase_withLoc
    = "test.ptolemy.util.TestOrderedResourceBundle";

  private final String propsFileBase_noLoc
    = "test.ptolemy.util.TestOrderedResourceBundleNoLoc";

  private final String[] expectedKey
    = new String[]{"key1", "key2", "key3", "key 4",
      "key:5", "key!6!value!6", "key#7#value#7", "key[8]"};

  private final String[] expectedVal
    = new String[]{"value1", "value2=doublevalue", "value3", "value 4",
      "value:5", "", "", "value[8]"};

  private OrderedResourceBundle orderedResourceBundle = null;

  public OrderedResourceBundleTest(String name) {
    super(name);
  }

  protected void setUp() throws Exception {
    super.setUp();

    assertNotNull("Resource Bundle File base name null - ", propsFileBase_withLoc);
    try {
      orderedResourceBundle = OrderedResourceBundle.getBundle(propsFileBase_withLoc);
    } catch (Exception ex) {
      fail("unexpected exception: " + ex);
    }
    assertNotNull("Ordered Resource Bundle - ", orderedResourceBundle);
  }

  protected void tearDown() throws Exception {
    orderedResourceBundle = null;
    super.tearDown();
  }

  public void testGetBundle() throws IOException {

    assertNotNull("Resource Bundle File base name null - ", propsFileBase_withLoc);

    OrderedResourceBundle actualReturn = null;
    try {
      actualReturn = OrderedResourceBundle.getBundle(
        propsFileBase_withLoc);
    } catch (Exception ex) {
      fail("unexpected exception: " + ex);
    }
    assertNotNull("return value - ", actualReturn);


    /////////////////////
    actualReturn = null;
    try {
      actualReturn = OrderedResourceBundle.getBundle(
          "test.ptolemy.util.TestOrderedResourceBundleNoLoc");
    } catch (Exception ex) {
        ex.printStackTrace();
      fail("unexpected exception: " + ex);
    }
    assertNotNull("return value - ", actualReturn);


    /////////////////////
    actualReturn = null;
    try {
      actualReturn = OrderedResourceBundle.getBundle(
          "test/ptolemy/util/TestOrderedResourceBundle");
    } catch (Exception ex) {
        ex.printStackTrace();
      fail("unexpected exception: " + ex);
    }
    assertNotNull("return value - ", actualReturn);


    /////////////////////

    actualReturn = null;
    try {
      actualReturn = OrderedResourceBundle.getBundle(
          "test/ptolemy/util/TestOrderedResourceBundleNoLoc");
  } catch (Exception ex) {
        ex.printStackTrace();
      fail("unexpected exception: " + ex);
    }
    assertNotNull("return value - ", actualReturn);


    /////////////////////


    actualReturn = null;
    Exception actualEx = null;

    try {
      actualReturn = OrderedResourceBundle.getBundle(null);
    } catch (Exception ex) {
      actualEx = ex;
    }
    assertTrue(
      "incorrect exception thrown - should be NullPointerException, but was "
      + actualEx, actualEx instanceof java.lang.NullPointerException);
    assertNull("return value should be null, but was: "
               + actualReturn, actualReturn);

    actualReturn = null;
    actualEx = null;
    try {
      actualReturn = OrderedResourceBundle.getBundle("BOGUS");
    } catch (Exception ex) {
      actualEx = ex;
    }
    assertTrue(
      "incorrect exception thrown - should be MissingResourceException, but was "
      + actualEx, actualEx instanceof java.util.MissingResourceException);
    assertNull("return value should be null, but was: "
               + actualReturn, actualReturn);
  }

  public void testGetKeys() {

    Iterator actualReturn = orderedResourceBundle.getKeys();
    int actualTotKeys = 0;
    String nextKey = null;
    while (actualReturn.hasNext()) {
        nextKey = (String)actualReturn.next();
        assertTrue("too many keys! ", expectedKey.length > actualTotKeys);
        assertEquals("wrong key - expected: " + expectedKey[actualTotKeys]
                     + ", but was: " + nextKey,
                     expectedKey[actualTotKeys], nextKey);
        actualTotKeys++;
    }

    assertEquals("return value", expectedKey.length, actualTotKeys);
  }

  public void testGetString() {

    for (int i = 0; i < expectedKey.length; i++) {

      String actualReturn = orderedResourceBundle.getString(expectedKey[i]);
      assertEquals("return value", expectedVal[i], actualReturn);
      assertEquals("wrong key - expected: " + expectedVal[i]
                   + ", but was: " + actualReturn, expectedVal[i], actualReturn);
    }

    String actualReturn = null;
    actualReturn = orderedResourceBundle.getString(null);
    assertNull("return value should be null", actualReturn);

    actualReturn = null;
    actualReturn = orderedResourceBundle.getString("BOGUS");
    assertNull("return value should be null", actualReturn);


  }

}
