/**
 *    '$RCSfile: WorkflowExecutionTestSuite.java,v $'
 *
 *     '$Author: brooke $'
 *       '$Date: 2006/05/11 22:28:08 $'
 *   '$Revision: 1.1 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


package test.workflows;

import junit.framework.*;


/**
 * <p>Test Suite that creates a test for each of the MOML workflows named in
 * the comma-delimited list contained in the environment variable "-DWF_LIST=".
 * This list is typically created and passed by ant
 *
 * @author Matthew Brooke
 * @version 1.0
 */
public class WorkflowExecutionTestSuite extends TestCase {

  private static String wfList;
  private static String[] workflows;

  public WorkflowExecutionTestSuite(String s) {
    super(s);
  }


  public static Test suite() {


    wfList = System.getProperty("WF_LIST");

    assertNotNull("workflow list (from JVM arg \"-DWF_LIST=\") is NULL ",
                  wfList);

    assertTrue("workflow list (from JVM arg \"-DWF_LIST=\") is EMPTY ",
               !wfList.trim().equals(""));

    workflows = wfList.split(",");

    assertNotNull("workflows array (split from workflows list) is NULL ",
                  workflows);
    assertTrue("workflows array (split from workflows list) is EMPTY ",
               workflows.length > 0);

    TestSuite suite = new TestSuite();

    for (int i = 0; i < workflows.length; i++) {
      System.out.println("Adding Test to Suite for Workflow: " + workflows[i]);
      WorkflowExecutionTest nextTest = new WorkflowExecutionTest("testWorkflow");
      nextTest.setWorkflowName(workflows[i]);
      suite.addTest(nextTest);
    }
    return suite;
  }
}
