/* AuthActor actor for testing Kepler authentication framework.

 Copyright (c) 1998-2003 The Regents of the University of California.
 All rights reserved.
 Permission is hereby granted, without written agreement and without
 license or royalty fees, to use, copy, modify, and distribute this
 software and its documentation for any purpose, provided that the above
 copyright notice and the following two paragraphs appear in all copies
 of this software.

 IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

 THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 ENHANCEMENTS, OR MODIFICATIONS.

                                        PT_COPYRIGHT_VERSION_2
                                        COPYRIGHTENDKEY
*/

package org.kepler.authentication.test;

import ptolemy.actor.TypedAtomicActor;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.*;
import ptolemy.actor.IOPort;
import ptolemy.actor.TypedIOPort;
import ptolemy.data.Token;
import ptolemy.data.StringToken;
import ptolemy.data.type.BaseType;
import ptolemy.data.type.Type;

import org.kepler.authentication.*;

//////////////////////////////////////////////////////////////////////////
//// AuthActor
/**
 * The AuthActor actor prompts the user to login into one GAMA server, 
 * and outputs the user's credential as a string from its output port.
 * This actor is a demonstration actor showing how to use Kepler 
 * authentication framework to get users' credentials for the workflow.
 * 
 * @author Zhijie Guan  
 * @version $Id: AuthActor.java,v 1.2 2006/05/19 00:53:56 guan Exp $
*/

public class AuthActor extends TypedAtomicActor {

	/** Construct AuthActor source with the given container and name.
	 *  @param container The container.
	 *  @param name The name of this actor.
     *  @exception IllegalActionException If the entity cannot be contained
     *   by the proposed container.
     *  @exception NameDuplicationException If the container already has an
     *   actor with this name.
     */
	
    public AuthActor(CompositeEntity container, String name)
            throws NameDuplicationException, IllegalActionException  {
        super(container, name);
        
        // initialize the output port
        output = new TypedIOPort(this, "output", false, true);
        output.setDisplayName("User's credential");
        output.setTypeEquals(BaseType.STRING);
        
        _attachText("_iconDescription", "<svg>\n" +
                "<rect x=\"0\" y=\"0\" "
                + "width=\"60\" height=\"20\" "
                + "style=\"fill:white\"/>\n" +
                "</svg>\n");
    }

    ///////////////////////////////////////////////////////////////////
    ////                     ports and parameters                  ////

    /** 
     * The user's credential is passed out of the actor through this port.
     */
    public TypedIOPort output = null;
    
    ///////////////////////////////////////////////////////////////////
    ////                         public methods                    ////

    /** Retrieve the credential from the AuthenticationManager and send
     *  it to the output port.
     *  @exception IllegalActionException If it is thrown by the
     *   send() method sending out the token.
     */
    public void fire() throws IllegalActionException {
        super.fire();
        output.send(0, new StringToken( aManager.getProxy().getCredential() ));
    }


    /**
     * private variables
     * aManager is an instance of AuthenticationManager
     */
	private AuthenticationManager aManager = AuthenticationManager.getManager();		// Authentication Manager

}
