/* GriddlesExec actor subclased from stanard Exec from PtolemyII concepts.
 Copyright (c) 2005 The Regents of the University of California.
 All rights reserved.

  Permission is hereby granted, without written agreement and without
 license or royalty fees, to use, copy, modify, and distribute this
 software and its documentation for any purpose, provided that the above
 copyright notice and the following two paragraphs appear in all copies
 of this software.

 IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

 THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 ENHANCEMENTS, OR MODIFICATIONS.
                                        PT_COPYRIGHT_VERSION_2
                                        COPYRIGHTENDKEY
*/

package org.monash.griddles;

import ptolemy.actor.IOPort;
import ptolemy.actor.TypedIOPort;
import ptolemy.data.Token;
import ptolemy.data.StringToken;
import ptolemy.data.type.BaseType;
import ptolemy.data.type.Type;
import ptolemy.data.expr.Parameter;
import ptolemy.actor.lib.Exec;
import ptolemy.actor.TypedAtomicActor;
import ptolemy.kernel.CompositeEntity;
import ptolemy.data.expr.StringParameter;
import ptolemy.kernel.util.NameDuplicationException;
import ptolemy.kernel.util.InternalErrorException;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.actor.parameters.PortParameter;
import ptolemy.kernel.util.Settable;
import ptolemy.kernel.util.Attribute;
import ptolemy.kernel.util.Workspace;


import ptolemy.data.BooleanToken;
import ptolemy.data.IntToken;
import ptolemy.data.expr.FileParameter;


import javax.xml.rpc.Stub;
import org.monash.griddles.jobrun.*;
import java.io.InputStream;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.File;
import java.io.BufferedReader;
import java.io.FileReader;

import org.w3c.dom.Comment;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.Source;
import javax.xml.transform.Result;
import javax.xml.transform.dom.DOMSource; 
import javax.xml.transform.stream.StreamResult; 
import java.io.FileOutputStream;


import java.io.ByteArrayOutputStream;
import java.util.HashSet;
import java.util.Hashtable;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.JLabel;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import au.edu.monash.csse.nimrod.NimrodJobInfo;

import com.jcraft.jsch.ChannelExec;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.Session;
import com.jcraft.jsch.UserInfo;
import com.jcraft.jsch.UIKeyboardInteractive;




/**
  *   GriddlesExec is a subclass of the GriddlesExec actor in 
  *   which we split functionality of above actor into two parts and 
  *   merged one part into web service implementation. We added web 
  *   service client functionality into this actor, which allows 
  *   communication between this actor and the web service. This actor 
  *   could be configured from the Kepler user interface for executing 
  *   command at any specified remote location. In addition a certificate 
  *   based security component has been implemented to make communication 
  *   between this actor and the web service fully secure. This actor encodes 
  *   the data with the public key of the web service and signs the data with 
  *   its private key before sending a request to the web service. The web 
  *   service on the other end opens the request by using a client public key 
  *   and decodes by using its private key. The web service executes the command 
  *   with the arguments that were received from the client. The stderr and stdout 
  *   messages generated by the execution are sent in response to the request. 
  *   The response is also encoded and signed with certificates as before. 
  *   The actor upon receiving response opens, decodes and creates stderr and stdout 
  *   objects. Subsequently these messages are streamed to respective ports like previous actor.
  *
  * 
  * This actor defines a new TypedIOPort for subclasses to use especially JGridletCreate ...   
  * @author Jagan Kommineni, Monash University July 2005
  * @version $Id: GriddlesExec.java,v 1.5 2006/05/09 02:56:09 kommineni Exp $
  **/


public class GriddlesExec extends Exec 
  {
  public GriddlesExec(CompositeEntity container, String name)
    throws NameDuplicationException, IllegalActionException  
    {
    super(container, name);
    
            // initialize our variables
    _jsch = new JSch();
    _setIdentities = new HashSet();
  
    userName = new Parameter(this, "userName", new StringToken("jagan"));    
    hostName = new Parameter(this, "hostName", new StringToken("brecca-2.vpac.org"));
    hostPort = new Parameter(this, "hostPort", new StringToken("33755"));
    modelReference= new Parameter(this,"modelReference", new StringToken(""));
    
    userName.setStringMode(true);
    hostName.setStringMode(true);
    hostPort.setStringMode(true);      

    streamingMode = new Parameter(this, "streaming mode", new BooleanToken(false));
    streamingMode.setTypeEquals(BaseType.BOOLEAN);
    
    
    identity = new FileParameter(this, "identity");  
    identity.setTypeEquals(BaseType.STRING);
            
    provider = new Parameter(this, "provider");
    _addProviderChoice();
 
    command.setToken(new StringToken("/bin/hostname "));
    //cmdArgs = new Parameter(this, "cmdArgs", new StringToken("Enter command line arguements")); 
    cmdArgs = new Parameter(this, "cmdArgs", new StringToken("")); 
    cmdArgs.setStringMode(true);
    provider.setStringMode(true);
    modelReference.setStringMode(true);
    }    
    ///////////////////////////////////////////////////////////////////
    ////                        public methods                     ////

    /** Callback for changes in attribute values
     *  Get the file or url selected.
     *
     *  @param a The attribute that changed.
     *  @exception IllegalActionException If the offsets array is not
     *   nondecreasing and nonnegative.
     */
  public void attributeChanged(Attribute at) throws IllegalActionException 
    {
    } // end of attributeChanged
    
  public StringParameter newStringParameter(String name) throws NameDuplicationException 
    {
    try 
      {
      _workspace.getWriteAccess();
      StringParameter param = new StringParameter(this, name);
      return param;
      } 
    catch (IllegalActionException ex) 
      {
      // This exception should not occur, so we throw a runtime
      // exception.
      // throw new InternalErrorException(this, ex, null);
      } 
    finally 
      {
      _workspace.doneWriting();
      }
    return null;
    }    
  /** Create a new TypedIOPort with the specified name.
  *  The container of the port is set to this actor.
  *  This method is write-synchronized on the workspace.
  *
  *  @param name The name for the new port.
  *  @return The new port.
  *  @exception NameDuplicationException If the actor already has a port
  *   with the specified name.
  */
    
  public ptolemy.kernel.Port newPort(String name) throws NameDuplicationException 
    {
    try 
      {
      _workspace.getWriteAccess();
      TypedIOPort port = new TypedIOPort(this, name);
      port.setTypeEquals(BaseType.STRING);
      return port;
      } 
    catch (IllegalActionException ex) 
      {
      // This exception should not occur, so we throw a runtime
      // exception.
      throw new InternalErrorException(this, ex, null);
      } 
    finally 
      {
      _workspace.doneWriting();
      }
    }  
  public Parameter userName;

      /**
     * The file path for <i>userName</i>'s ssh identity
     *    file if the user wants to connect without having to enter the
     *    password all the time.
     *
     *    <P> The user can browse this file as it is a parameter.
     */

  public FileParameter identity;
    
  public Parameter hostName;
  public Parameter hostPort;
  public Parameter provider;
  public Parameter cmdArgs;
  public Parameter modelReference;
     /**
     * Specifying whether the output should be sent
     *   in a streaming mode.
     */

  public Parameter streamingMode;
 
  private static Stub createProxy() 
    {
    // Note: MyJobrun_Impl is implementation-specific.
    return (Stub)(new MyJobrun_Impl().getJobrunIFPort());
    }
        
     /****** 
     The web service stub calls the web service to execute a command on remote 
     location and gets back stderr and stdout messages and these messages are
     streamed to the error and output ports respectively
     Web serves and clients developed by using jwsdp from sun .....
     *******/
  public synchronized void fire() throws IllegalActionException 
    {
    try
      {       
      String exHost =    (((StringToken)hostName.getToken()).stringValue()).trim();
      String exPort =    (((StringToken)hostPort.getToken()).stringValue()).trim();
      String jprovider =  (((StringToken)provider.getToken()).stringValue()).trim();
      String jcommand =   (((StringToken)command.getToken()).stringValue()).trim();
      String jarguments = (((StringToken)cmdArgs.getToken()).stringValue()).trim();
      String strUser = (((StringToken) userName.getToken()).stringValue()).trim();
              
      GriddlesUtil griddlesUtil = new GriddlesUtil();
      //NimrodClient nc = new NimrodClient();
                     
      if(jprovider.equalsIgnoreCase("jwsdp"))
        {
        String serviceURL="http://"+exHost+":"+exPort+"/jrs/services/JobrunIFPort";
        System.out.println("Service URL=" + serviceURL);      
        Stub stub = createProxy();

        stub._setProperty(javax.xml.rpc.Stub.ENDPOINT_ADDRESS_PROPERTY, serviceURL);
        JobrunIF jobrun = (JobrunIF)stub;
        String commandStr = jcommand+" "+jarguments;
  
        String stdErrorOut = jobrun.jobrun(commandStr);
        //   System.out.println("Error message:"+ stdErrorOut.getStderror());
        //   System.out.println("Output message:"+ stdErrorOut.getStdout());
        if(stdErrorOut == null)
          stdErrorOut ="%-%No results returned from server";
        System.out.println("ErrorOut="+stdErrorOut);
        String[] stringArray = stdErrorOut.split("%-%");        
        error.send(0, new StringToken(stringArray[0]));
        output.send(0, new StringToken(stringArray[1]));
        }
      else if(jprovider.equalsIgnoreCase("ssh"))
        {
        boolean isPassAuth = false;
        String strCommand = new String(jcommand+" "+jarguments);
        //String strHost = ( (StringToken) host.getToken()).stringValue();
        streaming = ((BooleanToken) streamingMode.getToken()).booleanValue();
        String strIdentity = ((StringToken) identity.getToken()).stringValue();

        if(strIdentity!=null && strIdentity.length()>0 ) 
          {
	  // Hack the path because we can't deal with "file:" or "file://"
	  if(strIdentity.startsWith("file:")) 
            {
	    strIdentity = strIdentity.substring(5);
            if(strIdentity.startsWith("//")) 
              {
	      strIdentity = strIdentity.substring(2);
	      }
	    }
          } 
        else 
          {
          //We are now need to connect with password
          isPassAuth = true;
          }
        try 
          {
          if(isPassAuth) 
            {
            //Use password authentication, where use will be prompted
            //for password. The password should be valid for the session
            _connect(strUser, exHost);  
            } 
          else 
            {
            _connect(strUser, exHost, strIdentity);
            }
          _exec(strCommand);
          } 
        catch(IllegalActionException e) 
          {
          // caught an exception, so output it to the errors port
          output.send(0, new StringToken(""));
          error.send(0, new StringToken(""));
          }
        }
      else if(jprovider.equalsIgnoreCase("nimrod"))
        {  
        NimrodClient nc = new NimrodClient();
        nc.args0 = new String("http://"+exHost+":"+exPort+"/NimrodService/jaxrpc/Nimrod");
        NimrodJobInfo nji = null;
        nji = nc.addCommand("addCommand keplerdemo "+jcommand+" "+jarguments);                     
        while(true)
          {
          nji = nc.getJobInfo(System.getProperty("user.name"), "getjobinfo keplerdemo "+nji.jobname);
          if("done".equalsIgnoreCase(nji.status))
            break;
          else
            Thread.sleep(3000l);
          }        
        error.send(0, new StringToken(nji.error));
        output.send(0, new StringToken(nji.output)); 
        }
      else if((jprovider.equalsIgnoreCase("gt2")) || (jprovider.equalsIgnoreCase("gt4")))
        {
        long currTime = System.currentTimeMillis()/1000; 
        String file_prefix ="g_"+currTime;
        String stdout1 = file_prefix+"_out";
        String stderr1 = file_prefix +"_err";
        String xmlScript= file_prefix+"_xml.xml";         
        griddlesUtil.createXMLFile(file_prefix, exHost, jprovider, jcommand, jarguments);         
        String kepler = System.getProperty("KEPLER");
        File kfile = new File(kepler+""+File.separator+"cog");   
        String classPath = griddlesUtil.visitAllDirsAndFiles(kfile);   
        System.out.println("classpatj="+classPath);
        classPath += kfile.pathSeparator;
        classPath += System.getProperty("java.class.path","."); 
        String wfCommnad =  "java -classpath "+classPath+" org.globus.cog.karajan.Loader "+ xmlScript;
        //System.out.println(wfCommnad);
        griddlesUtil.executeCommand(wfCommnad, null);  
        String stdOutStr="";
        String stdErrStr="";
        String line ;
        File oFile = new File(stdout1);
        BufferedReader br = null;
        if(oFile.exists())
          {                  
          br = new BufferedReader(new FileReader(oFile));  
          while ( (line=br.readLine()) != null ) 
            { 
            stdOutStr += line;
            stdOutStr += "\n";                 
            } 
          br.close();
          oFile.delete();  
          }
        File eFile = new File(stderr1);
        if(eFile.exists())
          {       
          br = new BufferedReader(new FileReader(eFile));  
          while ( (line=br.readLine()) != null ) 
            { 
            stdErrStr += line;
            stdErrStr += "\n";                 
            } 
          br.close();
          eFile.delete();          
          }
        File xFile = new File(xmlScript);
        if(xFile.exists())
          xFile.delete();     
        error.send(0, new StringToken(stdErrStr));
        output.send(0, new StringToken(stdOutStr)); 
        System.out.println("The process is successfully completed  ");
        }
      else 
        {
        long currTime = System.currentTimeMillis()/1000;
        String file_prefix ="g_"+currTime; 
        String stdout1 = file_prefix+"_out";
        String stderr1 = file_prefix+"_err";
       
        griddlesUtil.executeCommand(jcommand+" "+jarguments, file_prefix);
      
        String stdOutStr="";
        String stdErrStr="";
        String line ;
        BufferedReader br =null;
        File oFile = new File(stdout1);
        if(oFile.exists())
          {     
          br = new BufferedReader(new FileReader(oFile));  
          while ( (line=br.readLine()) != null ) 
            { 
            stdOutStr += line;
            stdOutStr += "\n";                 
            } 
          br.close();
          oFile.delete();
          }
        File eFile = new File(stderr1);  
        if(eFile.exists())
          {
          br = new BufferedReader(new FileReader(eFile));  
          while((line=br.readLine()) != null ) 
            { 
            stdErrStr += line;
            stdErrStr += "\n";                 
            } 
          br.close();
          eFile.delete();
          }
        error.send(0, new StringToken(stdErrStr));
        output.send(0, new StringToken(stdOutStr));                       
        }                  
      }
    catch(Exception ex) 
      { 
      System.out.println("The exception raised the message "+ex.getMessage());        
      ex.printStackTrace();      
      }
    }
  ///////////////////////////////////////////////////////////////////
  ////                         public methods                    ////
  //This method stops the execution at the end of first iteration  
  public boolean postfire() 
    {
    return false;
    }
  private void _addProviderChoice()
    {
    provider.addChoice("gt2");
    provider.addChoice("gt4");        
    provider.addChoice("jwsdp");        
    provider.addChoice("local");      
    provider.addChoice("nimrod");
    provider.addChoice("ssh");        
    }

 ////////////////// Private Methods ///////////////////////

 /**
  * @throws IllegalActionException If the connection fails.
  */
  private void _connect(String strUser, String strHost, String strIdentity)
        throws IllegalActionException 
    {

    _debug("Connecting with " + strUser + "@" + strHost + " with identity: " + strIdentity);
    try 
      {
      strIdentity = strIdentity.trim();
      if(!strIdentity.equals("")) 
        {
        if(_setIdentities.add(strIdentity)) 
          {
          // we haven't seen this identity before
           _jsch.addIdentity(strIdentity);
          }
        }
      if(!strUser.equals(_strOldUser) || !strHost.equals(_strOldHost) || !_session.isConnected()) 
        {
        if(null != _session && _session.isConnected()) 
          {
          _disconnect();
          }

        _session = _jsch.getSession(strUser, strHost, 22);
        _strOldUser = strUser;
        _strOldHost = strHost;

                // username and passphrase will be given via UserInfo interface.
        UserInfo ui = new MyUserInfo();
        _session.setUserInfo(ui);
        _session.connect(30000);
        }
      } 
    catch(Exception e) 
      {
      // a couple of possible exception messages that could happen here:
      // 1. java.io.FileNotFoundException
      // 2. session is down
      System.err.println("Exception caught in " + this.getFullName());
      System.err.println("I was trying to connect with " + strUser + "@"
                           + strHost + " with identity: " + strIdentity);
      e.printStackTrace();
      throw new IllegalActionException("Exception caught in " +
             this.getFullName() + "\n(" + e.getClass().getName() + ")\n"
                + e.getMessage());
      }
    }

    /**
     * Connect with password
     * 1. When connected for the first time, it will prompt for
     *   password.
     * 2. When execute for the same user@host, it should use the
     * 	 stored password.
     * 3. When connect to a different user@host, it can prompt password again.
     * @throws IllegalActionException If the connection fails.
     */
  private void _connect(String strUser, String strHost)
        throws IllegalActionException 
    {

    _debug("Connecting with " + strUser + "@" + strHost + " with password.");
    try 
      {
      if(!strUser.equals(_strOldUser) || !strHost.equals(_strOldHost) || !_session.isConnected()) 
        {
        if(null != _session && _session.isConnected()) 
          {
	  _disconnect();
          }
        _session = _jsch.getSession(strUser, strHost, 22);
        _strOldUser = strUser;
        _strOldHost = strHost;

        // username and passphrase will be given via UserInfo interface.

       //check whether ui is already set
       UserInfo ui;
       ui = (UserInfo)hash.get(strUser+"@"+strHost);
       if(ui == null) 
         {
         ui = new MyUserInfo();
         }
      //If it is already there. We will use that.
     //Hopefully we can use the info for connect to the
    //same user@host
       _session.setUserInfo(ui);
       _session.connect();
       //add to the hashtable
       hash.put(strUser+"@"+strHost, ui);
       }
     } 
   catch(Exception e) 
     {
     // a couple of possible exception messages that could happen here:
     // 1. java.io.FileNotFoundException
     // 2. session is down
     System.err.println("Exception caught in " + this.getFullName());
     System.err.println("I was trying to connect with " + strUser + "@"
                            + strHost + " with password.");
     e.printStackTrace();
     throw new IllegalActionException("Exception caught in " +
                this.getFullName() + "\n(" + e.getClass().getName() + ")\n"
                + e.getMessage());
     }
   }

   /**
    *
    * @throws IllegalActionException if disconnect fails.
    */
  private void _disconnect() throws IllegalActionException 
    {
    if(null == _session) 
      {
      // no session, so nothing to disconnect
      return;
      }
    try 
      {
      _session.disconnect();
      } 
    catch(Exception e) 
      {
      System.err.println("Exception caught in " + this.getFullName());
      e.printStackTrace();
      throw new IllegalActionException("Exception caught in " +
                this.getFullName() + "\n(" + e.getClass().getName() + ")\n"
                + e.getMessage());
      }
    }

    /**
     *
     * @throws IllegalActionException
     */
  private void _exec(String execCommand) throws IllegalActionException 
    {
    if(null == _session) 
      {
      // no session, so way to execute
      return;
      }
    try 
      {
      final ChannelExec channel = (ChannelExec) _session.openChannel("exec");
      channel.setCommand(execCommand);
      streamOut = new ByteArrayOutputStream();
      ByteArrayOutputStream streamErr = new ByteArrayOutputStream();

      channel.setOutputStream(streamOut);
      channel.setErrStream(streamErr);
      channel.connect();

      // wait for it to finish
      _thread = new Thread() 
        {
        public void run() 
          {
          int offset = 0;
          String current = "";
          while (!channel.isEOF()) 
            {
            if(_thread == null) 
              {
              return;
              }
            try 
              {
              sleep(500);
              if(streaming) 
                {
                //System.out.println(streamOut.size());
	        byte[] stream = streamOut.toByteArray();
	        int len = stream.length;
	        current += new String(stream,offset,len-offset);
	        //System.out.println(current);
		offset = len;
		current = _sendStreamOutput(current);
		//stdout.send(0,new StringToken(current));
                }
              }
            catch (Exception e) 
              {
              System.out.println(e.getMessage());
              }
            }
          try 
            {
            if(streaming) 
              {
	      byte[] stream = streamOut.toByteArray();
	      int len = stream.length;
	      current += new String(stream,offset,len-offset);
	      //System.out.println(current);
	      offset = len;
	      current = _sendStreamOutput(current);
	      if(!current.equals("")) 
                {
	         // there was some output that wasn't sent yet.
		 output.send(0,new StringToken(current));
		}
              }
            } 
          catch (Exception ex) {}
            }   
          };
       _thread.start();
       _thread.join(0); // set this to a different value to time out
      if(_thread.isAlive()) 
        {
        // ran out of time
        _thread = null;
        throw new IllegalActionException("In " + this.getFullName() + ": Remote operation timed out!");
        }
        // completed successfully
        // this is the wrong test if the remote OS is OpenVMS,
        // but there doesn't seem to be a way to detect it.
        int ec = channel.getExitStatus();
        channel.disconnect();
       if(!streaming) 
         {
         output.send(0, new StringToken(streamOut.toString()));
         }
       error.send(0, new StringToken(streamErr.toString()));
       } 
     catch(Exception e) 
       {
       System.err.println("Exception caught in " + this.getFullName());
       e.printStackTrace();
       throw new IllegalActionException("Exception caught in " +
       this.getFullName() + "\n(" + e.getClass().getName() + ")\n"
              + e.getMessage());
       }
    }

    /** This function streams the output line by line.
     *  If the current stream doesn't contain any line breaks and it is longer
     *  then a certain threshold, sends the stream content. Otherwise, if there
     *  is no line end, then it returns the content of the last line.
     *
     * @param currentStream
     * @return
     */
  public String _sendStreamOutput(String currentStream) throws IllegalActionException 
    {
    String line;
    int crInd;
    while ( (crInd = currentStream.indexOf("\n")) > -1) 
      {
      line = currentStream.substring(0,crInd);
      //System.out.println(line);
      try 
        {
    	currentStream = currentStream.substring(crInd+1);
    	} 
      catch(Exception ex) 
        {
    	// reached end of string.
    	//System.out.println(currentStream);
    	currentStream = "";
    	}
      output.send(0,new StringToken(line));
      }
    // if the whole output is a single line, stream it once the string
    // length reaches a certain threshold.
    if(currentStream.length() > 2048) 
      {
      output.send(0,new StringToken(currentStream));
      } 
    else 
      {
      // this line doesn't reach the threshold and contains no line break
      // it will be concatenated to the further output.
      return currentStream;
      }
    return "";
    }

    ////////////////// Private variables ///////////////////////

  private JSch _jsch = null;
  private Session _session = null;
  private Thread _thread = null;
  private HashSet _setIdentities = null;
  private String _strOldUser = null;
  private String _strOldHost = null;

  //////////////////Public Static ///////////////////////
  //Used to store connection info
  public static Hashtable hash = new Hashtable();

  ////////////////// Inner classes ///////////////////////

  public static class MyUserInfo implements UserInfo, UIKeyboardInteractive
    {
    public String getPassword() 
      {
      return passwd;
      }

    String passwd =null;
    JTextField passwordField = (JTextField) new JPasswordField(20);
    public boolean promptYesNo(String str) 
      {
      // This method gets called to answer the question similar to
      // "are you sure you want to connect to host whose key
      // is not in database ..."
      return true;
      }

    public String getPassphrase() 
      {
      return null;
      }

    public boolean promptPassphrase(String message) 
      {
      return false;
      }

    public boolean promptPassword(String message) 
      {
      if(passwd != null) 
        {
        return true;
        }
      Object[] ob = { passwordField };
      int result = JOptionPane.showConfirmDialog(null, ob, message, JOptionPane.OK_CANCEL_OPTION);
      if(result == JOptionPane.OK_OPTION) 
        {
        passwd = passwordField.getText();
        return true;
        } 
      else 
        {
        return false;
        }
      }
    public void showMessage(String message) 
      {
      // This method gets called when the server sends over a MOTD.
      //JOptionPane.showMessageDialog(null, message);
      }

    //
    // Extensions for supporting keyboard-interactive logins
    // Norbert Podhorszki pnorbert@cs.ucdavis.edu
    // Taken from example
    //     http://www.jcraft.com/jsch/examples/UserAuthKI.java
    //

    final GridBagConstraints gbc = new GridBagConstraints(0,0,1,1,1,1, GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0,0,0,0),0,0);
    private Container panel;
    public String[] promptKeyboardInteractive(String destination, String name,  String instruction, String[] prompt, boolean[] echo)
      {

	 //System.out.println("SSH: promptKI called\n"+
	 //	       "\tDestination: " + destination +
	 //	       "\n\tName: " + name +
	 //	       "\n\tinstruction: " + instruction +
	 //	       "\n\tpromptlen: " + prompt.length);

	 panel = new JPanel();
	 panel.setLayout(new GridBagLayout());

	 gbc.weightx = 1.0;
	 gbc.gridwidth = GridBagConstraints.REMAINDER;
	 gbc.gridx = 0;
	 panel.add(new JLabel(instruction), gbc);
	 gbc.gridy++;

	 gbc.gridwidth = GridBagConstraints.RELATIVE;

	 JTextField[] texts=new JTextField[prompt.length];
	 for(int i=0; i<prompt.length; i++)
           {
	   gbc.fill = GridBagConstraints.NONE;
	   gbc.gridx = 0;
	   gbc.weightx = 1;
	   panel.add(new JLabel(prompt[i]),gbc);

	   gbc.gridx = 1;
	   gbc.fill = GridBagConstraints.HORIZONTAL;
	   gbc.weighty = 1;
	 if(echo[i])
           {
	    texts[i]=new JTextField(20);
	    }
	  else
            {
	    texts[i]=new JPasswordField(20);
	    }
	  panel.add(texts[i], gbc);
	  gbc.gridy++;
	  }
        if(JOptionPane.showConfirmDialog(null, panel, destination+": "+name, JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE)
	       ==JOptionPane.OK_OPTION){
		String[] response=new String[prompt.length];
	for(int i=0; i<prompt.length; i++)
          {
	  response[i]=texts[i].getText();
	  }
	return response;
	}
      else
        {
	return null;  // cancel
	}
      }
    }
  private ByteArrayOutputStream streamOut;
  private boolean streaming = false;            
  }