/* Translating a moml description into actor relations axioms.

  Copyright (c) 2005 The Regents of the University of California.
 All rights reserved.

  Permission is hereby granted, without written agreement and without
 license or royalty fees, to use, copy, modify, and distribute this
 software and its documentation for any purpose, provided that the above
 copyright notice and the following two paragraphs appear in all copies
 of this software.

 IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

 THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 ENHANCEMENTS, OR MODIFICATIONS.

                                        PT_COPYRIGHT_VERSION_2
                                        COPYRIGHTENDKEY
                                        
*/

package org.monash.griddles;

import ptolemy.actor.CompositeActor;
import ptolemy.actor.Director;
import ptolemy.actor.IOPort;
import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedCompositeActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.actor.gui.BrowserLauncher;
import ptolemy.data.StringToken;
import ptolemy.data.Token;
import ptolemy.data.expr.FileParameter;
import ptolemy.data.expr.StringParameter;
import ptolemy.data.expr.Parameter;
import ptolemy.data.expr.Variable;
import ptolemy.data.type.BaseType;
import ptolemy.data.type.Type;
import ptolemy.data.type.TypeLattice;
import ptolemy.graph.CPO;
import ptolemy.graph.Inequality;
import ptolemy.graph.InequalityTerm;
import ptolemy.gui.MessageHandler;
import ptolemy.kernel.ComponentEntity;
import ptolemy.kernel.ComponentRelation;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.Port;
import ptolemy.kernel.Relation;
import ptolemy.kernel.util.*;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NamedObj;
import ptolemy.moml.MoMLParser;
import ptolemy.moml.Vertex;
import ptolemy.actor.lib.Exec;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
//for hash table .......
import java.util.Hashtable;
import java.util.Dictionary;
import java.util.Enumeration;

//jax rpc stuff ......

import javax.xml.rpc.Stub;

//////////////////////////////////////////////////////////////////////////
////JMakeGNSEntries
/**
 * Parsing the moml file into a container using MoMLParser. Then extracting the
 * schema infomration.
 *
 * and updating griddles name service ........
 * from this a connection between an output and an input port can be inferred.
 *
 * @author Jagan Kommineni 
 * @version $Id: JMakeGNSEntries.java,v 1.2 2006/05/09 02:56:09 kommineni Exp $
 * @since Ptolemy II 4.0.1
 *
*/

public class JMakeGNSEntries extends TypedAtomicActor
  {
  /** Construct an actor with the given container and name.
   *  @param container The container.
   *  @param name The name of this actor.
   *  @exception IllegalActionException If the actor cannot be contained
   *   by the proposed container.
   *  @exception NameDuplicationException If the container already has an
   *   actor with this name.
   */


  public JMakeGNSEntries(CompositeEntity container, String name)
    throws NameDuplicationException, IllegalActionException  
    {
    super(container, name);
    hostName = new Parameter(this, "hostName", new StringToken("brecca-2.vpac.org"));
    hostPort = new Parameter(this, "hostPort", new StringToken("33755"));
    hostName.setStringMode(true);
    hostPort.setStringMode(true);      

    proxy = new Parameter(this, "proxy");
    _addProxyChoice();
    
     config = new Parameter(this, "config");
    _addConfigChoice();
    
    proxy.setStringMode(true);
    config.setStringMode(true);
    
    outport = new TypedIOPort(this, "outport", false, true);
    outport.setTypeEquals(BaseType.STRING);
    
    _relationHash = new Hashtable();
    _portTypeHash = new Hashtable();
    _parameterHash = new Hashtable();
    _relationAttrHash = new Hashtable();
    
    
    }    
      
  public Parameter hostName;
  public Parameter hostPort;
  public Parameter config;
  public Parameter proxy;

  public TypedIOPort outport;
  
  

  ///////////////////////////////////////////////////////////////////
  ////                         public methods                    ////  
  // public void fire()
  public synchronized void fire() throws IllegalActionException   
    {  
    try 
      {
      String gns_data="";    
      boolean withProxy = false;
      int newConfig = 2;
              
      String configType = ((StringToken)config.getToken()).stringValue();
      String proxyType =  ((StringToken)proxy.getToken()).stringValue();
              
      if(configType.equalsIgnoreCase("use manually modified configuration") == true)
        newConfig = 3;
      if(configType.equalsIgnoreCase("create new configuration") == true)
        newConfig = 4;
              
      if(configType.equalsIgnoreCase("yes") == true)
        withProxy = true;
      
      NamedObj cont = this.getContainer();
       
       while(cont != null)
         {
         NamedObj cont1 = cont.getContainer();
         if(cont1 == null)
           break;
         cont = cont1;         
         }
       CompositeActor CE = (CompositeActor) cont;

	_getParams(CE);
	String entities = getEntities(CE);
     
         for (Enumeration e = _relationHash.keys() ; e.hasMoreElements() ;) 
          {                          
          String key = (String)e.nextElement();
          String value = (String)_relationHash.get(key.trim());
          String dataTransferMode="file";
          String dataFlowAttr = (String)_relationAttrHash.get(key.trim());
          if(dataFlowAttr != null)
            {
            if((dataFlowAttr.startsWith("srb"))||(dataFlowAttr.startsWith("\"srb")))
              dataTransferMode = "srb";
            else if((dataFlowAttr.startsWith("web"))||(dataFlowAttr.startsWith("\"web")))
              dataTransferMode = "soap";
            else if((dataFlowAttr.startsWith("grid"))||(dataFlowAttr.startsWith("\"grid")))
              dataTransferMode = "grid";
            else
              dataTransferMode="file";
            }            
          String relPort[] = value.split(",");          
          String portType0 =  (String)_portTypeHash.get(relPort[0]);
          String portType1 =  (String)_portTypeHash.get(relPort[1]);
          String rw0,rw1;
          if(portType0.equalsIgnoreCase("out"))
             rw0 = "W";     
          else
            {
            if(modelDirectorName.equalsIgnoreCase("SDF Director"))
               rw0 = "T";
            else
              rw0 = "R";      
            }
          if(portType1.equalsIgnoreCase("in"))
            {
            if(modelDirectorName.equalsIgnoreCase("SDF Director"))
              rw1 = "T";
            else
              rw1 = "R";       
            }               
          else
            rw1 = "W"; 
          String file0 =null;
          String file1 =null; 

          String actorName0 = relPort[0].substring(0, relPort[0].lastIndexOf("."));
          String actorName1 = relPort[1].substring(0, relPort[1].lastIndexOf("."));
          
          String modelReference0 = (String)_parameterHash.get(actorName0+".modelReference");
          String modelReference1 = (String)_parameterHash.get(actorName1+".modelReference");         
          
         if(modelReference0 == null)
           modelReference0 = "modelReference0";
      
         if(modelReference1 == null)
           modelReference1 = "modelReference1";            
      
         if(modelReference0 != null)
           {
           if(modelReference0.equalsIgnoreCase("GriddlesRemoteFile"))  
             file0 =  (String)_parameterHash.get(actorName0+".filename");
           else
             file0 =  (String)_parameterHash.get(relPort[0].trim());
           }
           
          if(modelReference1 != null)
           { 
           if(modelReference1.equalsIgnoreCase("GriddlesRemoteFile"))  
             file1 =  (String)_parameterHash.get(actorName1+".=:filename");
           else
             file1 =  (String)_parameterHash.get(relPort[1].trim());
           }
         if(file0 != null)
           {
           file0 = file0.replaceAll("\"", "");       
           file0.trim();           
           }
         if(file1 != null)
           {
           file1 = file1.replaceAll("\"", "");             
           file1.trim();
           }
            
          String executionHost0 = (String)_parameterHash.get(actorName0+".hostName"); 
          String executionHost1 = (String)_parameterHash.get(actorName0+".hostName"); 
          
          String webServicePort0 = (String)_parameterHash.get(actorName0+".hostPort"); 
          String webServicePort1 = (String)_parameterHash.get(actorName1+".hostPort");
             
          if(rw0.equalsIgnoreCase("W"))
            {
            if(executionHost0 == null || executionHost1 == null)
              {
              //gns_data += modelReference0+":"+executionHost0+":"+file0+" "+executionHost1+":"+webServicePort1+":"+file1+":"+rw0+":-:";
              //gns_data += modelReference1+":"+executionHost1+":"+file1+" "+executionHost1+":"+webServicePort1+":"+file1+":"+rw1+":-:";       
              continue;      
              }
              //System.out.println("executionHost0"+executionHost0+" executionHost1="+executionHost1);
                    
            if((executionHost0.equalsIgnoreCase(executionHost1) == false) &&  withProxy)
              {
              if(file0 == null || file1 == null)
                continue;
              String proxyInFile = file0+"_input";
              String proxyOutFile = file0+"_output";            
              if(modelDirectorName.equalsIgnoreCase("SDF Director"))
                {  
                gns_data += modelReference0+":"+executionHost0+":"+file0+" "+dataTransferMode+":"+executionHost0+":"+webServicePort0+":"+file0+":"+rw0+":PS:-:";
                gns_data += "PROXY:"+executionHost0+":"+proxyInFile+" "+dataTransferMode+":"+executionHost0+":"+webServicePort0+":"+file0+":T:-:";
                gns_data += "PROXY:"+executionHost0+":"+proxyOutFile+" "+dataTransferMode+":"+executionHost1+":"+webServicePort1+":"+file1+":"+rw0+":-:";                
                }
              else if(modelDirectorName.equalsIgnoreCase("PN Director"))
                {                
                gns_data += modelReference0+":"+executionHost0+":"+file0+" "+dataTransferMode+":"+executionHost0+":"+webServicePort0+":"+file0+":"+rw0+":PP:-:";
                gns_data += "PROXY:"+executionHost0+":"+proxyInFile+" "+dataTransferMode+":"+executionHost0+":"+webServicePort0+":"+file0+":R:-:";
                gns_data += "PROXY:"+executionHost0+":"+proxyOutFile+" "+dataTransferMode+":"+executionHost1+":"+webServicePort1+":"+file1+":"+rw0+":-:";                            
                }
              else
                {
                gns_data += modelReference0+":"+executionHost0+":"+file0+" "+dataTransferMode+":"+executionHost1+":"+webServicePort1+":"+file1+":"+rw0+":-:";
                }
              if(modelReference1.equalsIgnoreCase("GriddlesRemoteFile") == false)  
                gns_data += modelReference1+":"+executionHost1+":"+file1+" "+dataTransferMode+":"+executionHost1+":"+webServicePort1+":"+file1+":"+rw1+":-:";
              }
              else
              {                     
              if(file0 != null && file1 != null)
                {                  
                if((modelReference0 != null)&&(modelReference0 != null)&&(modelReference0.equalsIgnoreCase("GriddlesRemoteFile") == false))
                gns_data += modelReference0+":"+executionHost0+":"+file0+" "+dataTransferMode+":"+executionHost1+":"+webServicePort1+":"+file1+":"+rw0+":-:";
                }
              if((file1 != null)&&(modelReference1 != null)&&(modelReference1.equalsIgnoreCase("GriddlesRemoteFile") == false))
              gns_data += modelReference1+":"+executionHost1+":"+file1+" "+dataTransferMode+":"+executionHost1+":"+webServicePort1+":"+file1+":"+rw1+":-:";
              }
             }
           else
             {
             if(executionHost0 == null || executionHost1 == null)
               {
               //gns_data += modelReference1+":"+executionHost1+":"+file1+" "+executionHost0+":"+webServicePort0+":"+file0+":"+rw1+":-:";
               //gns_data += modelReference0+":"+executionHost0+":"+file0+" "+executionHost0+":"+webServicePort0+":"+file0+":"+rw0+":-:"; 
               continue;                         
               }
             if((executionHost0.equalsIgnoreCase(executionHost1) == false) &&  withProxy)
               {
               if(file0 == null || file1 == null)
                 continue;
               String proxyInFile = file1+"_input";
               String proxyOutFile = file1+"_output";                                
               if(modelDirectorName.equalsIgnoreCase("SDF Director"))
                 {    
                 gns_data += modelReference1+":"+executionHost1+":"+file1+" "+dataTransferMode+":"+executionHost1+":"+webServicePort1+":"+file1+":"+rw1+":PS:-:";
                 gns_data += "PROXY:"+executionHost1+":"+proxyInFile+" "+dataTransferMode+":"+executionHost1+":"+webServicePort1+":"+file1+":T:-:";
                 gns_data += "PROXY:"+executionHost1+":"+proxyOutFile+" "+dataTransferMode+":"+executionHost0+":"+webServicePort0+":"+file0+":"+rw1+":-:";
                 }
               else if(modelDirectorName.equalsIgnoreCase("PN Director"))
                 {
                 gns_data += modelReference1+":"+executionHost1+":"+file1+" "+dataTransferMode+":"+executionHost1+":"+webServicePort1+":"+file1+":"+rw1+":PP:-:";
                 gns_data += "PROXY:"+executionHost1+":"+proxyInFile+" "+dataTransferMode+":"+executionHost1+":"+webServicePort1+":"+file1+":R:-:";
                 gns_data += "PROXY:"+executionHost1+":"+proxyOutFile+" "+dataTransferMode+":"+executionHost0+":"+webServicePort0+":"+file0+":"+rw1+":-:";                     
                 }     
               else
                 gns_data += modelReference1+":"+executionHost1+":"+file1+" "+dataTransferMode+":"+executionHost0+":"+webServicePort0+":"+file0+":"+rw1+":-:";
               if(modelReference0.equalsIgnoreCase("GriddlesRemoteFile") == false)  
                 gns_data += modelReference0+":"+executionHost0+":"+file0+" "+dataTransferMode+":"+executionHost0+":"+webServicePort0+":"+file0+":"+rw0+":-:";
               }
             else
               {
               if(file0 != null && file1 != null)
                 {
                 if((modelReference1 != null)&&(modelReference1.equalsIgnoreCase("GriddlesRemoteFile") == false))        
                 gns_data += modelReference1+":"+executionHost1+":"+file1+" "+dataTransferMode+":"+executionHost0+":"+webServicePort0+":"+file0+":"+rw1+":-:";
                 }
               if((file0 != null)&&(modelReference0 != null)&&(modelReference0.equalsIgnoreCase("GriddlesRemoteFile") == false))
               gns_data += modelReference0+":"+executionHost0+":"+file0+" "+dataTransferMode+":"+executionHost0+":"+webServicePort0+":"+file0+":"+rw0+":-:";                     
               }
             }
           } 
          System.out.println("The director name="+modelDirectorName);
          System.out.println("Gns Data="+gns_data);
          outport.send(0, new StringToken(gns_data));   
          gns_update(gns_data, newConfig);
	  _toplevel = null;
          } 
        //catch (IOException io) {io.printStackTrace();
                
           catch (Exception ex) {ex.printStackTrace();     
          }
        }

     public String gns_update(String gns_data, int newConfig)
       {
       try
         {
         String  gns_host=    ((StringToken)hostName.getToken()).stringValue();
         String gns_port =    ((StringToken)hostPort.getToken()).stringValue();        
         String serviceURL = new String("http://"+gns_host+":"+gns_port+"/gns/services/GnsIFPort");       
         System.out.println("Service URL= " + serviceURL +"! for gns update");                                
         Stub stub = createProxy();
         stub._setProperty(javax.xml.rpc.Stub.ENDPOINT_ADDRESS_PROPERTY, serviceURL);
         org.monash.griddles.gns.GnsIF gns = (org.monash.griddles.gns.GnsIF)stub;
        
        String s1 = (String)(gns.getFileLocation(gns_data, newConfig));
        System.out.println(s1);
         }
      catch (Exception ex) { ex.printStackTrace(); }
     return null;
     }
   private static Stub createProxy() 
     {
     // Note: MyGns_Impl is implementation-specific.
     return (Stub)(new org.monash.griddles.gns.MyGns_Impl().getGnsIFPort());
     }

    /** Initialize global variables.
     */
   public void initialize()  
     {
     //super.initialize();        
     _container = "";
     _containment = "";
     }
   public boolean postfire() 
     {
     _container = "";
     _containment = "";
    return false;
    }

    
 ///////////////////////////////////////////////////////////////////
////                        private methods                     ////

    private void _addProxyChoice()
    {
    proxy.addChoice("yes");
    proxy.addChoice("no");   
    }
    
  private void _addConfigChoice()
    {
    config.addChoice("create new configuration");
    config.addChoice("update old configuration");
    config.addChoice("use manually modified configuration");
    } 
  private String getEntities(CompositeActor CE) 
    {
    Iterator ents = CE.entityList().iterator();
    String axiom = "";
    Director director = CE.getDirector();
    
    
    
    if (director == null) 
      {
      return "no top level director!";
      }
      else
       modelDirectorName = director.getName();
              
    Location locAtt = (Location) director.getAttribute("_location");
    String loc = locAtt.getExpression();
    _container = CE.getFullName();

    if (_container.length() > _workspaceLen) 
      {
       _container = _container.substring(_workspaceLen);
      }
    _getRelations(CE);

    while (ents.hasNext()) 
      {
      Object obj = ents.next();
      ComponentEntity CA = (ComponentEntity) obj;
      String container = CA.getContainer().getFullName();
      if (container.length() > _workspaceLen) 
        {
        container = container.substring(_workspaceLen);
        }
     _containment += "contains('" + container + "','";
     _containment += CA.getName() + "').\r\n";
     _getPorts(CA);
     _getParams(CA);
     if (CA instanceof TypedCompositeActor) 
       {
       CompositeActor inner = (CompositeActor) CA;
       axiom += getEntities(inner);
    	//TODO: add composite entity name??
    	}
      axiom += "actor('" + CA.getFullName().substring(_workspaceLen) + "','";
      axiom += obj.getClass().getName() + "').\r\n";
      locAtt = (Location) CA.getAttribute("_location");
      loc = locAtt.getExpression();
      }
    return axiom;
    }
  /** Returns all information about an actor's parameters */
  private void _getParams(ComponentEntity CE) 
    {
    String actorParams = new String("");
    String parameterValues = new String("");
    Iterator props = CE.attributeList().iterator();
    while (props.hasNext()) 
      {
      Attribute property =  (Attribute)props.next();
      String propName = property.getName();
      String attClass = property.getClass().getName();
      if (!propName.startsWith("_") && !attClass.toLowerCase().endsWith("director")) 
        {
	int ind = attClass.lastIndexOf(".");
	String shortClass = attClass.substring(ind + 1);
	String value = "";
        //System.out.println("shortClassName="+ shortClass+" property="+property+" attClass="+attClass);
	if (shortClass.startsWith("StringAtt") || shortClass.startsWith("FileAtt")) 
          {
	  StringAttribute stringAtt = (StringAttribute) property;
	  value = stringAtt.getExpression();
	  } 
        else if (shortClass.equals("Attribute")) 
          {
	  continue;
          } 
        else if (shortClass.equals("NamedObjId")) 
          {
	  continue;
	  } 
        else if (shortClass.equals("SemanticType"))
          {
           continue;
          }
        else if (shortClass.equals("DocumentationAttribute"))
          {
           continue;
          }                    
        else 
          {
          //System.out.println("shortClass="+shortClass+" property="+property);
          Variable param = (Variable) property;
	  value = param.getExpression();
	  }
	//TODO: ADD IF FULLNAME > WORKSPACE LEN!!
        //System.out.println("workspaceLen="+_workspaceLen+ " fullname="+CE.getFullName());
        String fullname = CE.getFullName();
        String actorName;
        boolean isRelationProperty = false;
        if(fullname.length() > _workspaceLen)
          {
	  actorName = CE.getFullName().substring(_workspaceLen);
          }
        else
          {
          isRelationProperty = true;
          actorName = CE.getFullName(); 
          }
        actorParams += "actorParmeter('";
	actorParams += actorName;
	actorParams += "','" + propName + "').\r\n";
	parameterValues += "parameterValue(" + actorName;
	//parameterValues += "." + propName + "," + attClass;
        parameterValues += "." + propName;
	parameterValues += "," + value + ")\r\n";
        
        if(isRelationProperty)
        {
        _parameterHash.put(propName.trim(),value);
        }
        else
        {
                String paramKey = actorName+"."+propName;
        _parameterHash.put(paramKey.trim(),value);
        }
	}
      }
    }

/** Returns all information about an actor's ports */
  private String _getPorts(ComponentEntity CE) 
    {
    String actorPorts = new String("");  
    String portIOType = new String("");   
    String portDataType = new String("");
    String portRange = new String("");  
    String typeConstraints = new String("");  
    String typeConstraintValues = new String("");
    Iterator ports = CE.portList().iterator();
    while (ports.hasNext()) 
      {
      TypedIOPort p = (TypedIOPort)ports.next();
      String actorName = CE.getFullName().substring(_workspaceLen);
      String portName = actorName + "." + p.getName();
      String tgStr, tlStr;
      Iterator typeConstraintsIter = p.typeConstraintList().iterator();
      while (typeConstraintsIter.hasNext()) 
        {
	Inequality ie = (Inequality)typeConstraintsIter.next();
	InequalityTerm tg = ie.getGreaterTerm();
	InequalityTerm tl = ie.getLesserTerm();
	NamedObj tgObj = (NamedObj) tg.getAssociatedObject();
	String tgClassName = tgObj.getClass().getName();
	String tgVarName = tgObj.getFullName().substring(_workspaceLen);
	NamedObj tlObj = (NamedObj) tl.getAssociatedObject();
	String tlClassName = tlObj.getClass().getName();
	String tlVarName = tlObj.getFullName().substring(_workspaceLen);
        typeConstraints += "typeConstraint('" + portName + "','";
	typeConstraints += tlClassName + "','" + tlVarName + "','";
	typeConstraints += tgClassName + "','" + tgVarName + ").\r\n";
	try 
          {
	  tlStr = tl.getValue().toString();
	  tgStr = tg.getValue().toString();
	  typeConstraintValues += "typeConstraintValue('" + portName + "','";
	  typeConstraintValues += tl.getValue() + "','" + tg.getValue() +"').\r\n";
          } 
        catch (IllegalActionException e) {e.printStackTrace();}
        }
      Type higher = BaseType.UNKNOWN;
      Type lower = BaseType.GENERAL;
      Iterator conPorts = p.connectedPortList().iterator();
      //The type range is between the lowest and highest.
      int compare;
      while (conPorts.hasNext()) 
        {
	TypedIOPort conP = (TypedIOPort)conPorts.next();
	Type conType = conP.getType();
	compare = TypeLattice.compare(conType,lower);
	if (compare == CPO.LOWER) 
          {
	  lower = conType;
	  }
        compare = TypeLattice.compare(higher, conType);
        if (compare == CPO.LOWER) 
          { 
          higher = conType;
	  }
	}
      compare = TypeLattice.compare(lower, higher);
      if (compare == CPO.HIGHER) 
        {
	higher = lower;
	}
      portRange += "portTypeRange('" + portName + "','";
      portRange += lower + "','" + higher + "').\r\n";
      actorPorts += "actorPorts('" + actorName + "','";
      actorPorts += portName + "').\r\n";                  
      String ioType = "";
      if (p.isInput() && p.isOutput()) 
        {
	ioType = "inOut";
	} 
      else if (p.isInput()) 
        {
	ioType = "in";
	} 
      else if (p.isOutput()) 
        {
	ioType = "out";
	} 
      else System.out.println("ERROR: no port IO type!");
      String pchannels = "singlePort";
      if (p.isMultiport()) 
        {
	pchannels = "multiPort";
	}
      portIOType += "portIOType(" + portName;
      portIOType += "," + ioType;
      portIOType += "," + pchannels + ")\r\n";
      _portTypeHash.put(portName, ioType);
      portDataType += "portDataType('" + portName;
      portDataType += "','" + p.getType() + "').\r\n";                       
      }
    return null;
    }
  private void _getRelations(CompositeEntity CE) 
    {
    String relations = new String("");
    String container = CE.getFullName();
    if (container.length() > _workspaceLen) 
      {
      container = container.substring(_workspaceLen);
       }
    Iterator rels = CE.relationList().iterator();
    while (rels.hasNext()) 
      {
      Relation rel = (Relation) rels.next();
      String relName = rel.getFullName().substring(_workspaceLen);
      String relShortName = rel.getName();
      String relClass = rel.getClass().getName();
      relations += "relation('" + relName + "','";
      relations += relClass + "').\r\n";
      String vertex = "";
      Iterator relAtts = rel.attributeList().iterator();
      while(relAtts.hasNext())
        {
        Attribute property =  (Attribute)relAtts.next();
        String propName = property.getName();

        String propClass = property.getClass().getName();
        if(propClass.equalsIgnoreCase("org.monash.griddles.GriddlesParameter"))
          {
          Variable param = (Variable) property;
	  String  value = param.getExpression();        
          System.out.println("relName.trim()="+relName.trim()+"value="+value);      
            _relationAttrHash.put(relName.trim(), value); 
            /*
          if(relShortName.equalsIgnoreCase(propName))
            {
            _relationAttrHash.put(relName.trim(), value);
            }
          else if(propName.equalsIgnoreCase("Parameter"))
            {
            System.out.println("relName.trim()="+relName.trim()+"value="+value);      
            _relationAttrHash.put(relName.trim(), value);        
            }
            */
          }
        }      
      Iterator ports = rel.linkedPortList().iterator();
      String links = new String("");
      while (ports.hasNext()) 
        {
    	Port p = (IOPort)ports.next();
    	String portName = p.getFullName().substring(_workspaceLen);
    		
        if(links.length() < 3)
          links += portName;
        else
          links += ","+portName;
    	}
       _relationHash.put(relName.trim(), links);                                      
       }
     }

    private NamedObj _parse(URL _momlUrl) 
      {
      MoMLParser parser = new MoMLParser();
      parser.reset();
      try 
        {
         
    	_toplevel = parser.parse(_momlUrl, _momlUrl);
        
    	} 
      catch (Exception ex)          
        {
        System.out.println("The exception error raised");
        System.out.println("moml parser exception" + ex.getMessage());	}
    	return _toplevel;
      }
    ///////////////////////////////////////////////////////////////////
    ////                         private members                   ////
    /** current entity's container */
    private String _container = "";
    /** the containment relation */
    private String _containment = "";
     /** top level entity */
    private NamedObj _toplevel = null;
     /** workspace name */
    private String _workspace;
    /** length of workspace name */
    private int _workspaceLen;
    //hash tables variables ....
    private String modelDirectorName = null;

    private Hashtable _relationHash = null;
    private Hashtable _relationAttrHash = null;
    private Hashtable _portTypeHash = null;
    private Hashtable _parameterHash = null;
    }